# **Mobroute:** CLI Userguide

[Mobroute](http://sr.ht/~mil/mobroute) is a general purpose FOSS public
transportation router (e.g. trip planner) Go library and CLI that works by
directly ingesting timetable (GTFS) data from transit agencies themselves
(sourced from the Mobility Database). For an overview of Mobroute's
functionality see [Mobroute's project page](http://sr.ht/~mil/mobroute).
This document is a guide on how to run Mobroute's CLI by compiling the
Go codebase and then running routing requests locally.

- *1:* [Primer Documentation](#strongprimer-documentationstrong)
- *2:* [Mobroute CLI: Install & Running Custom Routing Requests](#strongmobroute-cli-install-amp-running-custom-routing-requestsstrong)
- *3:* [Mobroute CLI: Subcommands Documentation](#strongmobroute-cli-subcommands-documentationstrong)
- *4:* [Further Resources & Documentation](#strongfurther-resources-amp-documentationstrong)


## **Primer Documentation**

- Before using Mobroute's CLI, please read the following documentation:
  - [Mobroute: Project Overview & Featureset](https://git.sr.ht/~mil/mobroute)
  - [Mobroute: What is MDBID and the Mobility Database Doc](https://git.sr.ht/~mil/mobroute/tree/master/doc/doc_mdbid.md)
  - [Mobroute: Algorithm Tunable Parameters](https://git.sr.ht/~mil/mobroute/tree/master/doc/doc_tunables.md)

## **Mobroute CLI: Install & Running Custom Routing Requests**

- (1) Installation assumes Go (>=1.18)
- (2) Clone the Mobroute repository:
  - `git clone http://git.sr.ht/~mil/mobroute`
- (3) Build the Mobroute binary:
  - `./build.sh`
- (4) Run your routing request by passing the request as YAML:
  - ```sh
./mobroute route -p '{
  load: {filter: {mdbid: [516]}},
  from: {lat: 40.70940, lon: -74.00537},
  to: {lat: 40.72879, lon: -73.95215},
  output_formats: [legs, mapurl]
}'
    ```
- (5) The route response is JSON:
  - See an [example output response here](http://ci.lrdu.org/tests_results/json/516.json)
  - The properties of `legs` and `mapurl` are included as those are the `output_formats`
    specified in the request
  - The `legs` format:
    - Shows the calculated route formatted into steps the user can take to accomplish the route
    - E.g. (board train x at y time etc.; transfer here or there in a more verbose format).
  - The `mapurl` format:
    - The route is converted to GeoJSON and displayed as a URL link to a [web geojson rendering service](http://geojson.lrdu.org))
    - See [an example of a route map format rendering here](http://geojson.lrdu.org/#%5B%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22walk%22%2C%22name%22%3A%22Walk+from+origin+%2840.7094%2C+-74.0054%29%3A+21%3A22%3A20+%E2%86%92+21%3A26+%283m+40s%29%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-74.0054%2C40.7094%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22board%22%2C%22name%22%3A%22Board+trip+%28A+toward+Ozone+Park-Lefferts+Blvd%29%3A+21%3A26+%E2%86%92+21%3A32%3A30+%286m+30s%29%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-74.0077%2C40.7102%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28High+St%29+at+21%3A29%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9905%2C40.6993%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Jay+St-MetroTech%29+at+21%3A31%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9873%2C40.6923%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Hoyt-Schermerhorn+Sts%29+at+21%3A32%3A30%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.985%2C40.6885%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22board%22%2C%22name%22%3A%22Transfer%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.985%2C40.6885%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22board%22%2C%22name%22%3A%22Board+trip+%28G+toward+Court+Sq%29%3A+21%3A37+%E2%86%92+21%3A51%3A30+%2814m+30s%29%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.985%2C40.6885%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Fulton+St%29+at+21%3A38%3A30%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9754%2C40.6871%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Clinton-Washington+Avs%29+at+21%3A40%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9668%2C40.6881%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Classon+Av%29+at+21%3A41%3A30%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9601%2C40.6889%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Bedford-Nostrand+Avs%29+at+21%3A42%3A30%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9535%2C40.6896%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Myrtle-Willoughby+Avs%29+at+21%3A44%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.949%2C40.6946%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Flushing+Av%29+at+21%3A45%3A30%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9502%2C40.7004%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Broadway%29+at+21%3A46%3A30%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9503%2C40.7061%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Metropolitan+Av%29+at+21%3A48%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9514%2C40.7128%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Nassau+Av%29+at+21%3A50%3A30%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9513%2C40.7246%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22stop%22%2C%22name%22%3A%22Trip+stop+%28Greenpoint+Av%29+at+21%3A51%3A30%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9544%2C40.7314%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22icon%22%3A%22walk%22%2C%22name%22%3A%22Walk+to+destination+%2840.7288%2C+-73.9522%29%3A+21%3A51%3A30+%E2%86%92+21%3A57%3A24+%285m+54s%29%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22Point%22%2C%22coordinates%22%3A%5B-73.9522%2C40.7288%5D%7D%7D%2C%7B%22type%22%3A%22Feature%22%2C%22properties%22%3A%7B%22name%22%3A%22Route%22%7D%2C%22geometry%22%3A%7B%22type%22%3A%22LineString%22%2C%22coordinates%22%3A%5B%5B-74.0054%2C40.7094%5D%2C%5B-74.0077%2C40.7102%5D%2C%5B-74.0077%2C40.7102%5D%2C%5B-74.0077%2C40.7102%5D%2C%5B-73.9905%2C40.6993%5D%2C%5B-73.9873%2C40.6923%5D%2C%5B-73.985%2C40.6885%5D%2C%5B-73.985%2C40.6885%5D%2C%5B-73.985%2C40.6885%5D%2C%5B-73.985%2C40.6885%5D%2C%5B-73.985%2C40.6885%5D%2C%5B-73.985%2C40.6885%5D%2C%5B-73.9754%2C40.6871%5D%2C%5B-73.9668%2C40.6881%5D%2C%5B-73.9601%2C40.6889%5D%2C%5B-73.9535%2C40.6896%5D%2C%5B-73.949%2C40.6946%5D%2C%5B-73.9502%2C40.7004%5D%2C%5B-73.9503%2C40.7061%5D%2C%5B-73.9514%2C40.7128%5D%2C%5B-73.9513%2C40.7246%5D%2C%5B-73.9544%2C40.7314%5D%2C%5B-73.9544%2C40.7314%5D%2C%5B-73.9544%2C40.7314%5D%2C%5B-73.9522%2C40.7288%5D%5D%7D%7D%5D)

## **Mobroute CLI: Subcommands Documentation**

You can retrieve documentation about each mobroute subcommand via using
`-h`. For example `mobroute route -h` yields:

```sh
Usage of route:
  -d string
        Verbose debug categories: (i) Information (w) Warn (d) Debug (default "i")
  -dprof string
        Write debug pprof CPU profile to file
  -format string
        Output format, currently only json supported (default "json")
  -p string
        YAML Parameters for db subcommand; note JSON is also valid YAML as well

        Valid properties:
          - filter: Object based on mobsql's filter specification, see git.sr.ht/~mil/mobsql (default: {mdbid: [516]})
          - from: Object with two float properties, lat & lon (default: {lat: 40.70940, lon: -74.00537})
          - to: Object with two float properties, lat & lon (default: {lat: 40.72879, lon: -73.95215})
          - time: Datetime to depart from trip, if unset defaults to current time / now (default: null)
          - max_walk_seconds: Maximum seconds walking distance on start/end of trip to get to/from origin/destination stops (default: 1200)
          - max_trip_seconds: Maximum seconds the entire route/trip can take (default: 14400)
          - min_transfer_seconds: Minimum seconds each transfer in the route can take (default: 180)
          - max_transfer_seconds: Maximum seconds each transfer in the route can take (default: 2400)
          - max_n_transfers: Maximum number of transfers a route can contain (default: 20)
          - walkspeed_km_hr: Ratio of kilometers per hour the user walk (default: 4.5)
          - transfer_categories: Array of transfers categories to load - "f" for feed, "i" for implicit, "g" for generated (default: ["f", "i"])
          - cache_dir: Directory to store sqlite db, defaults to ~/.cache/mobroute if null (default: null)
          - output_formats: Array of formats for the output may include - "legs", "geojson", "diagnostics", "mapurl", "request" (default: ["legs"])
                 (default "{load: {filter: {mdbid: [516]}}, from: {lat: 40.70940, lon: -74.00537}, to: {lat: 40.72879, lon: -73.95215}, output_formats: [legs]}")
```


## **Further Resources & Documentation**

- [Automated Test Results / Metros](https://ci.lrdu.org/tests_results/): If you're curious about metros that have been well-tested and MDBIDs which you can experiment with for routing, this page has a number of working routing requests (lat/lon pairs and MDBIDs)
- [Development Guide](doc_dev.md): If you're curious about how Mobroute works under-the-hood, this guide can help architecture overview of what happens when you submit a routing request, from ETL, to algorithm, to formatting
- [Contribute Guide](doc_contribute.md): If you're intrested in contributing, this goes over the workflow.
- [Transito](http://git.sr.ht/~mil/transito): The Transito Mobile app runs the same routing logic the CLI executes on a mobile app (Android & Linux)

---

> [All Mobroute Docs](http://sr.ht/~mil/mobroute)
